package de.hanser.buch.opiz.domain;


import org.hibernate.Hibernate;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Beschreibt eine Bestellung eines Kunden.
 * <p>
 * Bei der Erzeugung hat die Bestellung den Status NEW.
 * Pizzen können der Bestellung hinzugefügt werden.
 * <p>
 * Wenn eine Bestellung fertig ist, bekommt es den Status OPEN.
 * Dieser Status ist relevant für die Pizza-Lieferant.
 * <p>
 * Nach Auslieferung der Bestellung bekommt sie den Status DELIVERED.
 * <p>
 *  
 */
@Entity
@Table(name = "orders", uniqueConstraints = @UniqueConstraint(columnNames = {
        "customer_fk", "creationtime" }))
// order ist ein SQL-Schlüsselwort und damit als Tabellenname nicht zulässig.
// {customer_fk, creatiomtime} ist der Business-Key für diese Klasse und muss
// deshalb unique sein.
public class Order implements Serializable {
    private Integer id;

    private Customer customer;

    private List<Pizza> pizzas = new ArrayList<Pizza>();

    // Eine neue Bestellung hat den Status NEW
    private OrderState state = OrderState.NEW;

    private Date creationTime = new Date();

    /**
     * Default-Konstruktor fuer Hibernate.
     */
    public Order() {
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    /**
     * Der Kunde, der diese Bestellung aufgegeben hat.
     */
    @ManyToOne(optional = true)
    @JoinColumn(name = "customer_fk")
    public Customer getCustomer() {
        return customer;
    }

    public void setCustomer(Customer customer) {
        this.customer = customer;
    }

    /**
     * Die Pizzen dieser Bestellung. <p/> Demonstriert eine
     * Parent/Child-Beziehung mit cascade = CascadeType.ALL
     */
    @OneToMany(mappedBy = "order", cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    public List<Pizza> getPizzas() {
        return pizzas;
    }

    public void setPizzas(List<Pizza> pizzas) {
        this.pizzas = pizzas;
    }

    /**
     * Order-Zustand - eine Enumeration. <p/> Demonstriert die
     * Enumerated-Annotation.
     */
    @Enumerated(EnumType.STRING)
    public OrderState getState() {
        return state;
    }

    public void setState(OrderState aState) {
        this.state = aState;
    }

    /**
     * Der Preis der Bestellung ist der Summe der Preise der einzelnen Pizzen.
     * <p/> Der Preis wird berechnet, deswegen Transient-Annotation.
     * 
     * @return der Preis der Bestellung.
     */
    @Transient
    public double getPrice() {
        double price = 0.0;
        for (Pizza pizza : pizzas) {
            price += pizza.getPrice();
        }
        return price;
    }

    /**
     * Fuegt eine Pizza der Bestellung hinzu.
     * <p>
     * Nur möglich, wenn die Bestellung den Status NEW hat.
     * 
     * @param pizza
     *            die Pizza
     * @return FALSE falls die Bestellung den falschen Status hat, sonst TRUE.
     */
    public boolean addPizza(Pizza pizza) {
        if (state != OrderState.NEW) {
            return false;
        }
        pizzas.add(pizza);
        pizza.setOrder(this);
        return true;
    }

    /**
     * Entfernt eine Pizza aus einer Bestellung.
     * 
     * @param pizza
     *            die Pizza
     */
    public void removePizza(Pizza pizza) {
        pizzas.remove(pizza);
        pizza.setOrder(null);
    }

    /**
     * Der Erstellungszeitpunkt der Bestellung. <p/> Wird automatisch
     * initialisiert. <p/> Demonstriert die Temporal-Annotation.
     */
    @Temporal(TemporalType.TIMESTAMP)
    public Date getCreationTime() {
        return creationTime;
    }

    public void setCreationTime(Date creationTime) {
        this.creationTime = creationTime;
    }

    /**
     * Demonstriert die Implementation von equals auf der Basis eines
     * zusammengesetzten Business Keys.
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || Hibernate.getClass(this) != Hibernate.getClass(o)) return false;

        final Order order = (Order) o;

        if (creationTime != null ? !creationTime
                .equals(order.creationTime)
                : order.creationTime != null) return false;
        if (customer != null ? !customer.equals(order.customer)
                : order.customer != null) return false;

        return true;
    }

    /**
     * Demonstriert die Implementation von hashcode auf der Basis eines
     * zusammengesetzten Business Keys.
     */
    @Override
    public int hashCode() {
        int result;
        result = (customer != null ? customer.hashCode() : 0);
        result = 29
                * result
                + (creationTime != null ? creationTime.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return "id: " + id + ", state: " + state + ", pizzas: "
                + pizzas + ", customer: " + customer
                + ", creationTime: " + creationTime;
    }

    public boolean containsPizza(int aPizzaId) {
        for (Pizza pizza : this.pizzas) {
            if (pizza.getId() == aPizzaId) {
                return true;
            }
        }
        return false;
    }

    /**
     * Aendere den Order-State von NEW auf OPEN.
     * @return FALSE falls der Order den falschen State hat, sonst TRUE.
     */
    public boolean finishOrder() {
        boolean stateChangedOK = false;
        if (state == OrderState.NEW) {
            state = OrderState.OPEN;
            stateChangedOK = true;
        }
        return stateChangedOK;
    }

}
