package de.hanser.buch.opiz.web.jsf;

import java.util.Locale;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.acegisecurity.userdetails.UserDetails;
import org.springframework.context.MessageSource;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.transaction.annotation.Transactional;

import de.hanser.buch.opiz.auth.CustomerAuthService;
import de.hanser.buch.opiz.dao.CustomerDao;
import de.hanser.buch.opiz.domain.Address;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.service.CustomerService;
import de.hanser.buch.opiz.web.RequestAndSessionKeyConstants;

/**
 * Aktionen rund um Kunden.
 */
@Transactional(readOnly = false)
public class CustomerAction  {

    private CustomerService customerService;
    
    private CustomerAuthService customerAuthService;

    private CustomerDao customerDao;
    
    private MessageSource messageSource;
    
    /**
     * Registriert einen Kunden im System.
     * 
     * @return Aktion
     */
    @SuppressWarnings("unchecked")
    public String register() {
        FacesContext ctx = FacesContext.getCurrentInstance();
        Customer customer = (Customer) ctx.getExternalContext().getRequestMap().get(
                RequestAndSessionKeyConstants.REQUEST_KEY_CUSTOMER);
        Address address = (Address) ctx.getExternalContext().getRequestMap().get(
                RequestAndSessionKeyConstants.REQUEST_KEY_ADDRESS);
        customer.setAddress(address);
        try {
            
            customerService.createCustomer(customer);
            UserDetails userDetails = customerAuthService.getUserDetails(customer);
            customerAuthService.authenticate(userDetails, customer.getPassword());
        } catch (DataIntegrityViolationException e) {
        	Locale locale = FacesContext.getCurrentInstance().getViewRoot().getLocale();
            String errorTxt = messageSource.getMessage("Customer.nonUnique", null, locale);
            FacesContext.getCurrentInstance().addMessage(null,
                    new FacesMessage(FacesMessage.SEVERITY_ERROR, errorTxt, e.getMessage()));
            return null;
        }

        return "register";
    }

    /**
     * Speichert einen (genderten) Kunden.
     * 
     * @return Aktion
     */
    @SuppressWarnings("unchecked")
    public String saveCustomer() {
        FacesContext ctx = FacesContext.getCurrentInstance();
        Customer customer = (Customer) ctx.getExternalContext().getRequestMap().get(
                RequestAndSessionKeyConstants.REQUEST_KEY_CUSTOMER);
        Address address = (Address) ctx.getExternalContext().getRequestMap().get(
                RequestAndSessionKeyConstants.REQUEST_KEY_ADDRESS);
        Customer currentCustomer = getCurrentCustomer();

        currentCustomer.setName(customer.getName());
        currentCustomer.setPassword(customer.getPassword());
        currentCustomer.getAddress().setStreet(address.getStreet());
        currentCustomer.getAddress().setZip(address.getZip());
        currentCustomer.getAddress().setCity(address.getCity());
        
        customerDao.updateCustomer(currentCustomer);
        return "home";
    }

    /**
     * Bereitet das Editieren eines Kunden vor.
     * 
     * @return Aktion
     */
    @SuppressWarnings("unchecked")
    public String editCustomer() {
        FacesContext ctx = FacesContext.getCurrentInstance();
        Customer currentCustomer = getCurrentCustomer();
        Customer customer = new Customer();
        customer.setName(currentCustomer.getName());
        customer.setPassword(currentCustomer.getPassword());
        ctx.getExternalContext().getRequestMap().put(
                RequestAndSessionKeyConstants.REQUEST_KEY_CUSTOMER, customer);
        Address address = new Address();
        address.setStreet(currentCustomer.getAddress().getStreet());
        address.setZip(currentCustomer.getAddress().getZip());
        address.setCity(currentCustomer.getAddress().getCity());
        ctx.getExternalContext().getRequestMap().put(
                RequestAndSessionKeyConstants.REQUEST_KEY_ADDRESS, address);
    
        return "editCustomer";
    }
    
    /**
     * Liefert den aktuell angemeldeten Kunden.
     * 
     * @return der Kunde
     */
    public Customer getCurrentCustomer() {
        Customer customer = getCustomerFromHttpSession();
        
        // ====================================
        // Bitte auskommentieren, um eine LazyInitializationExceptions zu provozieren
        //customerDao.attachCustomer(customer);
        // ====================================
        
        return customer;
    }

    /**
     * Diese Methode dient der Abstraktion.
     * Im ersten Teil von Kapitel 9 ist nur wichtig, dass der Kunde aus der Http-Session
     * geholt wird. Dass der Kunde hier in einem Sicherheitskontext von Acegi liegt und
     * ber den AuthService geladen wird, spielt erst im Zusammenhang mit Acegi eine Rolle.
     * @return das Kundenobjekt aus der Http-Session
     */
    private Customer getCustomerFromHttpSession() {
        return customerAuthService.getPrincipal();
    }

    public void setCustomerService(CustomerService customerService) {
		this.customerService = customerService;
	}
    
    public void setCustomerAuthService(CustomerAuthService customerAuthService) {
        this.customerAuthService = customerAuthService;
    }
    
    public void setCustomerDao(CustomerDao customerDao) {
        this.customerDao = customerDao;
    }
    
    public void setMessageSource(MessageSource messageSource) {
        this.messageSource = messageSource;
    }
}
