package de.hanser.buch.opiz.dao;

import static de.hanser.buch.opiz.Constants.FUNC_LOG;
import static de.hanser.buch.opiz.Constants.LOG_MSG_ORDER_NEW_OR_CHANGED;

import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Restrictions;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.HibernateTemplate;

import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderFilter;
import de.hanser.buch.opiz.domain.OrderState;
import de.hanser.buch.opiz.domain.Pizza;

/**
 * Erweitert HibernateDaoSupport NICHT, als Beispiel.
 * Benutzt aber HibernateCallback und HibernateTemplate.
 */
public class HibernateOrderDaoImpl implements OrderDao {

    private HibernateTemplate hibernateTemplate;

    public void setHibernateTemplate(
            HibernateTemplate hibernateTemplate) {
        this.hibernateTemplate = hibernateTemplate;
    }

    /**
     * {@inheritDoc}
     */
    public Order getOrder(Integer id) {
        return (Order) hibernateTemplate.get(Order.class, id);
    }

    /**
     * {@inheritDoc}
     */
    public Integer saveOrder(Order anOrder) {
        hibernateTemplate.saveOrUpdate(anOrder);
        FUNC_LOG.info(LOG_MSG_ORDER_NEW_OR_CHANGED + anOrder.getId());
        return anOrder.getId();
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public List<Order> getOrdersInState(final OrderState state) {
        return (List<Order>) hibernateTemplate
                .execute(new HibernateCallback() {
                    public Object doInHibernate(Session session) {
                        Query query = session
                                .createQuery("from Order o where o.state=?");
                        query.setParameter(0, state);
                        return query.list();
                    }
                });
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public List<Order> getOrdersForCustomer(final Customer customer) {
        return (List<Order>) hibernateTemplate
                .execute(new HibernateCallback() {
                    public Object doInHibernate(Session session) {
                        Query query = session
                                .createQuery("from Order o where o.customer=?");
                        query.setParameter(0, customer);
                        return query.list();
                    }
                });
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public List<Order> selectOrder(OrderFilter filter) {
        Criteria criteria = this.hibernateTemplate
                .getSessionFactory().getCurrentSession()
                .createCriteria(Order.class);
        if (filter != null) {
            if (filter.getState() != null) {
                criteria.add(Restrictions.eq("state", filter
                        .getState()));
            }
            if ((filter.getPizzas() != null)
                    && (filter.getPizzas().size() != 0)) {
                // fuege Restriktion hinzu
                criteria.createCriteria("pizzas").add(
                        Restrictions.in("id", filter.getPizzas()));
            }
            if (filter.getCustomerName() != null) {
                // fuege Restriktion hinzu
                criteria.createCriteria("customer").add(
                        Restrictions.like("name", filter
                                .getCustomerName()));
            }
        }

        List<Order> orderList = criteria.list();

        return orderList;
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public List<Order> selectOrderByExample(Order exampleOrder) {
        Criteria criteria = this.hibernateTemplate
                .getSessionFactory().getCurrentSession()
                .createCriteria(Order.class);
        Example example = Example.create(exampleOrder).ignoreCase();

        criteria.add(example);
        // fuege sub-criteria hinzu
        if (exampleOrder.getCustomer() != null) {
            criteria.createCriteria("customer").add(
                    Example.create(exampleOrder.getCustomer())
                            .enableLike());
        }

        if ((exampleOrder.getPizzas() != null)
                && (exampleOrder.getPizzas().size() != 0)) {
            List<Integer> pizzaList = new ArrayList<Integer>();
            for (Pizza pizza : exampleOrder.getPizzas()) {
                pizzaList.add(pizza.getId());
            }
            criteria.createCriteria("pizzas").add(
                    Restrictions.in("id", pizzaList));
        }

        List<Order> orderList = criteria.list();

        return orderList;
    }

    /* **************************************************
     * Die folgende Methoden zeigen die Benutzung von
     * Stored-Procedures mit Jdbc, Spring, und Hibernate.
     * ************************************************** 
     */

    /**
     * Ein Beispiel, wie eine Stored-Procedure mit JDBC aufgerufen werden kann.
     * 
     * Zu benutzen: Code wieder einkommentieren
     * und den Oracle-Treiber ojdbc14.jar bereitstellen
     * (siehe book-libraries/lib/other/ojdbc14.jar.README)
     */
    
//    public ResultSet stpViaJdbc(OrderState state) throws SQLException {
//        CallableStatement stp = hibernateTemplate.getSessionFactory()
//                .getCurrentSession().connection().prepareCall(
//                        "{ ? = call orders.get_orders (?) }");
//
//        stp.registerOutParameter(1, OracleTypes.CURSOR);
//        stp.setInt(2, state.ordinal());
//        stp.execute();
//        return (ResultSet) stp.getObject(1);
//    }

    // stp via Spring: benutze GetOrdersSTP direkt

    public ResultSet stpViaHibernate(OrderState state) {
        return (ResultSet) hibernateTemplate.findByNamedQuery(
                "orders.get_orders", state.ordinal());
    }

}
