package de.hanser.buch.opiz.service;

import java.util.Calendar;

import org.apache.log4j.Logger;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;



/**
 * Der Aspekt stellt Hilfsmethoden zur Lagerbestandsverwaltung zur Verfuegung. 
 * @author Gerald Bachlmayr
 *
 */
@Aspect
public class InventoryChecker {
	
	private static int pizzaCounter = 100;
	private static int orderCounter = 0;
	private static final Logger LOG = Logger.getLogger(InventoryChecker.class);
	

    /**
     * Das Lager wird auf eine eingehende Bestellung vorbereitet.
     */
	@Before("execution(* de.hanser.buch.opiz.dao.*Order*.saveOrder(de.hanser.buch.opiz.domain.Order))")
	public void prepareStockForOrder() {
		// Geschaeftslogik hier:
		checkRecorderLevel();
		LOG.debug("Inventar - Pizzavorrat: " + pizzaCounter);
	}


    /**
     * Der Lagerbestand wird vermindert und die Anzahl der eingegangenen Bestellungen erhoeht.
     * @throws InventoryException tritt auf, wenn der Lagerbestand nicht reduziert werden kann.
     */
    @After("execution(* de.hanser.buch.opiz.dao.*Order*.saveOrder(de.hanser.buch.opiz.domain.Order))")
	public void reduceStock() throws InventoryException {
        if (pizzaCounter < 1) {
            throw new InventoryException("Der Lagerbestand kann nicht reduziert werden");
        }
		LOG.debug("Inventar - Alter Pizzavorrat: " + pizzaCounter);
		pizzaCounter--;
		orderCounter++;
		LOG.debug("Inventar - Neuer Pizzavorrat: " + pizzaCounter);
	}

    /**
     * FailOvers werden geloggt.
     *
     */
	@AfterThrowing("execution(* de.hanser.buch.opiz.dao.OrderDao.saveOrder(..))")
	public void doFailOver() {
		LOG.warn("hibFlushInterceptor.afterThrowing");
		LOG.debug("Inventar - Fehlerbehandlung");
	}
	
	public void countOrders() {
		orderCounter++;
		LOG.debug("Inventar - eine Bestellung mehr");
	}

    /**
     * Misst die Zeit des Bearbeitungsvorgangs.
     * @param pjp ProceedingJoinPoint
     * @return Das Zielobjekt wird zurueckgegeben
     * @throws Throwable wird beim Auftreten eines Fehlers geworfen
     */
	@Around("execution(* de.hanser.buch.opiz.dao.*Order*.saveOrder(de.hanser.buch.opiz.domain.Order))")
	public Object doTimer(ProceedingJoinPoint  pjp) throws Throwable {
		long startTime = Calendar.getInstance().getTimeInMillis();
		Object retVal = pjp.proceed();
		long endTime = Calendar.getInstance().getTimeInMillis();
		LOG.debug("Dauer: " + (endTime - startTime));
		return retVal;
	}
	
	private void checkRecorderLevel() {
		// Dummy-Methode: hier koennte der Meldebestand ueberprueft werden
	}


	public static int getOrderCounter() {
		return orderCounter;
	}


	public static void setOrderCounter(int orderCounter) {
		InventoryChecker.orderCounter = orderCounter;
	}


	public static void setPizzaCounter(int pizzaCounter) {
		InventoryChecker.pizzaCounter = pizzaCounter;
	}


	public static int getPizzaCounter() {
		return pizzaCounter;
	}
	
}
