package de.hanser.buch.opiz.service;

import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderFilter;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Interface fr einen Bestellservice. 
 *
 */
@Transactional(propagation=Propagation.REQUIRES_NEW, isolation=Isolation.REPEATABLE_READ, readOnly=true)
public interface OrderService {
		
    /**
     * Anlegen einer neuen Bestellung, die 1-n Pizzen und einen Kunden beinhaltet
	 * @param o die Bestellung
	 * @param custId die Kundennummer
	 */
	public void createOrder(Order o, Integer custId);
	
    /**
     * Ermittelt die Wartezeit einer Lieferung anhand der offenen Bestellungen. 
     * @return die Wartezeit in Minuten
     */
    public int getQueueTime();
    
    /**
     * Legt eine neue Order an.
     * 
     * @param aPizzaId
     *            id der ausgewaehlten Pizza.
     * @return ID der neuen Order.
     */
    @Transactional(readOnly = false)
    Integer createPendingOrder(Integer aPizzaId);

    /**
     * Fuegt eine Pizza einer bestehenden Order hinzu. Im Erfolgsfall wird nichts
     * zurueckgeliefert. Eine Fehler wird durch eine Exception gekennzeichnet.
     * <p>
     * Moeglicher Fehler: der Status der Bestellung muss NEW sein, sonst koennen
     * keine Pizzen der Bestellung hinzugefuegt werden.
     * 
     * @param anOrderId
     *            id der ausgewaehlten Order.
     * @param aPizzaId
     *            id der ausgewaehlten Pizza.
     * @return FALSE falls die Pizza nicht hinzugefuegt werden kann (z. B.
     *         falscher Order-Status), sonst TRUE.
     */
    @Transactional(readOnly = false)
    boolean addPizzaToOrder(Integer anOrderId, Integer aPizzaId);

    /**
     * Gibt den aktuellen Preis der Order zurueck.
     * 
     * @param anOrderId
     */
    double getPrice(Integer anOrderId);

    /**
     * Schliet eine Order ab. Danach sind keine Aenderungen mehr moeglich.
     * 
     * @param anOrderId
     * @return TRUE falls OK, sonst FALSE.
     */
    @Transactional(readOnly = false)
    boolean finishOrder(Integer anOrderId);

    /**
     * Gibt eine Liste von Order-Objekten zurueck, die dem Filter entsprechen.
     * 
     * @param filter
     *            das Filter-Objekt.
     * @return eine Liste von Order-Objekten.
     */
    List<Order> selectOrder(OrderFilter filter);

}

